/**
 * Selection jQuery plugin
 */
(function ($) {
	$.fn.selection = function () {
		var selection = {
			start: 0,
			end:   0
		};
		
		if (arguments.length == 0) {
			return get(this[0]);
		}
		
		if (arguments.length == 1 && typeof arguments[0] == 'object') {
			selection = arguments[0];
		} else {
			selection.start = arguments[0];
			if (arguments.length > 1) {
				selection.end = arguments[1];
			}
		}
		
		this.each(
			function (index, element) {
				set(element, selection);
			}
		);
		return this;
	};
	
	/**
	 * Private methods
	 */
	
	function get(element) {
		try {
			if (element.selectionStart != undefined) {
				return {
					start: element.selectionStart,
					end:   element.selectionEnd
				};
			} else if (document.selection && document.selection.createRange) {
				var range = document.selection.createRange();
				return {
					start: range.moveStart('character'),
					end:   range.moveEnd('character')
				};
			}
		} catch (err) {
			return {
				start: 0,
				end:   0
			};
		}
	}
	
	function set(element, selection) {
		try {
			if (element.setSelectionRange) {
				element.setSelectionRange(selection.start, selection.end);
			} else if (element.createTextRange) {
				var range = element.createTextRange();
				range.collapse(true);
				range.moveStart('character', selection.start);
				range.moveEnd('character', selection.end);
				range.select();
			}
		} catch (err) {
			
		}
	}
	
})(jQuery);
