
var keyCode = {
	ALT:              18,
	BACKSPACE:         8,
	CAPS_LOCK:        20,
	COMMA:           188,
	COMMAND:          91,
	COMMAND_LEFT:     91,
	COMMAND_RIGHT:    93,
	CONTROL:          17,
	DELETE:           46,
	DOWN:             40,
	END:              35,
	ENTER:            13,
	ESCAPE:           27,
	HOME:             36,
	INSERT:           45,
	LEFT:             37,
	MENU:             93,
	NUMPAD_ADD:      107,
	NUMPAD_DECIMAL:  110,
	NUMPAD_DIVIDE:   111,
	NUMPAD_ENTER:    108,
	NUMPAD_MULTIPLY: 106,
	NUMPAD_SUBTRACT: 109,
	PAGE_DOWN:        34,
	PAGE_UP:          33,
	PERIOD:          190,
	RIGHT:            39,
	SHIFT:            16,
	SPACE:            32,
	TAB:               9,
	UP:               38,
	WINDOWS:          91 
};

/**
 * Autocomplete
 */
(function ($) {
	$.fn.autocomplete = function () {
		return this.each(
			function (index, element) {
				var $text     = $(element),
					$dropdown = $('#'+escapeID($text.attr('dropdown'))),
					$value    = $('#'+escapeID($text.attr('for'))),
				
					initial,
					current = 0,
					
					keyTimeout,
					ajax
				;
				
				function initialState() {
					return {
						value: $value.val(),
						text: $text.val()
					};
				}
				
				initial = initialState();
				
				element.dropdown = function () {
					ajax = $.ajax({
						async: true,
						type: 'POST',
						data: $text.serialize()+'&action='+$text.attr('path')+':autocomplete',
						cache: false,
						context: $dropdown,
						success: function (data) {
							var $dropdown = $(this);
							$dropdown.html(
								$(data).find('#'+escapeID($dropdown.attr('id'))).html()
							);
							
							if ($dropdown.children().length > 0 && $text.is(':focus')) {
								$dropdown.show();
							}
							
							current = $dropdown.children('[value="'+$value.val()+'"]').index()+1;
							
							if (current == 0) {
								$value.val('');
								if (!$text.is(':focus')) {
									$text.val('');
									$text.change();
								}
							} else {
								$dropdown.children().eq(current-1).addClass('active');
							}
							
							$dropdown.children().mousedown(
								function () {
									var $active = $(this);
									$text.val($active.text());
									$value.val($active.attr('value'));
									$dropdown.children().removeClass('active');
									$active.addClass('active');
								}
							);
								
						}
					});
					
				};
				
				$text.
					unbind('.autocomplete').
					unbind('.state').
					
					bind('focus.autocomplete',
						function () {
							initial = initialState();
							
							if ($dropdown.children().length > 1) {
								$dropdown.show();
							}
						}
					).
					
					bind('change.autocomplete',
						function (event) {
							$text.data('lastTyped', $text.val());
							if (initial.value == $value.val()) {
								$text.val(initial.text);
								event.stopPropagation();
								event.stopImmediatePropagation();
							}
						}
					).
					
					bind('blur.autocomplete',
						function () {
							$dropdown.hide();
						}
					).
					
					state().
					
					bind('keydown.autocomplete',
						function (event) {
							var old = current;
							
							switch (event.which) {
								case keyCode.ALT :
								case keyCode.SHIFT :
								case keyCode.CONTROL :
								case keyCode.INSERT :
									
								case keyCode.TAB :
									
								case keyCode.LEFT :
								case keyCode.RIGHT :
								case keyCode.HOME :
								case keyCode.END :
									return true;
								break;
								
								case keyCode.ENTER :
									event.stopPropagation();
									$dropdown.hide();
									$text.change();
									return false;
								break;
								
								case keyCode.ESCAPE :
									$dropdown.hide();
									
									$value.val(initial.value);
									$text.val(initial.text);
									$dropdown.children().remove();
									current = 0;
									
									return true;
								break;
								
								case keyCode.UP : 
									current--;
									if (current < 1) {
										current = 1;
										return true;
									}
								break;
								
								case keyCode.DOWN :
									var len = $dropdown.children().length;
									current++;
									if (current > len) {
										current = len;
										return true;
									}
								break;
							}
							
							if (old != current) {
								$dropdown.show();
								$dropdown.children().removeClass('active');
								var $active = $dropdown.children().eq(current-1).addClass('active');
								
								$value.val($active.attr('value'));
								$text.val($active.text());
								
								return true;
							}
							
							if (keyTimeout) {
								clearTimeout(keyTimeout);
								keyTimeout = undefined;
							}
							
							if (ajax) {
								ajax.abort();
								ajax = undefined;
							}
							
							keyTimeout = setTimeout(
								function () {
									$text.dropdown();
								},
								500
							);

						}
					)
				;
				
			}
		);
	};
})(jQuery);

(function ($) {
	$.fn.dropdown = function () {
		return this.each(
			function (index, element) {
				if (element.dropdown) {
					element.dropdown();
				}
			}
		);
	};
})(jQuery);