function escapeID(str) {
	if (typeof str == 'string') {
        return str.replace(/([ #;?%&,.+*~\':"!^$[\]()=>|\/@])/g,'\\$1');
	}
	return str;
}

/**
 * AjaxConveyor jQuery plugin
 */
(function ($) {
	$.AjaxConveyor = function (settings) {
		return new AjaxConveyor(settings);
	};
	
	function AjaxConveyor(settings) {
		$.extend(this.settings, settings);
	}

	AjaxConveyor.prototype = {
		settings: {},
		
		options: new Array(),
		
		pipe: function (request) {
			this.options.push(request);
			this.run();
		},
		
		run: function () {
			if (this.options.length == 0) {
				if (this.doneCallback) {
					this.doneCallback();
				}
				return;
			}
			
			if (this.current) {
				switch (this.current.next) {
					case 'abort' :
						this.current.ajax.abort();
					break;
					default: return;
				}
			}
			
			var conveyor = this;
			
			var options  = this.options.shift();
			if (this.beforeCallback) {
				this.beforeCallback(options);
			}
			this.current = {
				next: options.next,
				ajax: $.ajax('/', options.ajax)
			};
			
			this.current.ajax.done(
				function () {
					delete(conveyor.current);
					conveyor.run();
				}
			);
		},
		
		before: function (callback) {
			this.beforeCallback = callback;
			return this;
		},
		
		done: function (callback) {
			this.doneCallback = callback;
			return this;
		}
	};
})(jQuery);

function ajaxResponse(context, data) {
	if (typeof data == 'string') {
		data = data.substring(data.indexOf('<body '));
	}
	
	var $data = $(data);
	var $context = $(context);
	
	if ($data.length == 1 && $data.is('script')) {
		eval(
			typeof window.XMLSerializer != 'undefined' ?
			$data.text()
			:
			$data.html()
		);
		return;
	}
	
	$data = $data.length == 1 && $data.is('#ajax-response') ? 
		$data.children()
		:
		($data.find('#ajax-response').length != 0 ?
			$data.find('#ajax-response').children()
			:
			$data
		)
	;
	
	$data.each(
		function (index, element) {
			var $element = $(element);
			
			if ($element.length == 1 && $element.is('script')) {
				eval(
					typeof window.XMLSerializer != 'undefined' ?
					$element.text()
					:
					$element.html()
				);
				return;
			}
			
			var id = $element.attr('id');
			var t = (id ?
				$('#'+escapeID(id))
				:
				null
			);
			
			var $target = (t && t.length ?
				t
				:
				$context
			);
			
			if ($element.hasClass('remove')) {
				$target.detach();
			} if ($element.hasClass('append')) {
				$target.append(
					$element.children().xml()
				);
			} else {
				var val, sel;
				var $focus = $(':focus');
				var focusID = $focus.attr('id');
				
				if (focusID) {
					val = $focus.val();
					sel = $focus.selection();
				}
				
				$target.replaceWith(
					$element.xml()
				);
				
				if (focusID) {
					$focus = $('#'+escapeID(focusID));
					$focus.focus();
					$focus.val(val);
					$focus.selection(sel);
				}
			}
			
		}
	);
	$(document).trigger('Ready');
}

/**
 * XML 
 */
(function ($) {
	var rclass = /[\n\t\r]/g;
	
	$.fn.extend({
		xml: function () {
			var xml = '';
			
			this.each(
				function (index, element) {
					if (typeof window.XMLSerializer != 'undefined') {
						xml += (new XMLSerializer()).serializeToString(element);
						return;
					}
					xml += (typeof element.xml != 'undefined' ?
						element.xml
						:
						(typeof element.outerHTML != 'undefined' ?
							element.outerHTML
							:
							''
						)
					);
				}
			);
			
			return xml;
		},
	
		hasClass: function ( selector ) {
			var className = " " + selector + " ",
			i = 0,
			l = this.length;
			for ( ; i < l; i++ ) {
				var err, current;
				try {
					current = (this[i].className !== undefined ?
						this[i].className
						:
						this[i].getAttribute('class')
					);
				} catch (err) {
					current = '-';
				}
				
				if ( this[i].nodeType === 1 && (" " + current + " ").replace(rclass, " ").indexOf( className ) > -1 ) {
					return true;
				}
			}
	
			return false;
		}
	});
})(jQuery);

(function ($) {
	$.modal = function (message) {
		$('#modal').remove();
		if (message == undefined) {
			return;
		}
		
		$('#page').append(
			'<div id="modal">'+
				(message === false ?
					'&nbsp;'
					:
					'<div class="container">'+
						'<p>'+message+'</p>'+
					'</div>'
				)+
			'</div>'
		);
	};
})(jQuery);


/**
 * Confirm
 */
(function ($) {
	$.fn.confirm = function () {
		return this.each(
			function (index, element) {
				var $this = $(element);
				
				$this.unbind('.confirm');
				
				$this.bind('click.confirm',
					function (event) {
						$.modal(false);
						if (confirm('Are you sure?')) {
							return true;
						}
						$.modal();
						event.stopImmediatePropagation();
						return false;
					}
				);
			}
		);
	};
})(jQuery);

/**
 * Actions
 */
(function ($) {
	$.fn.action = function () {
		return this.each(
			function (index, element) {
				var $this = $(element);
				var id    = $this.attr('id');
				var name  = $this.attr('name');
				
				var $form  = $this.closest('form');
				var upload = $form.attr('enctype') == 'multipart/form-data';
				
				var actionOnly = !$this.is(':submit') && $form.is('.extra');
				
				if ($this.is('.confirm')) {
					$this.confirm();
				}
				
				if (id == undefined && name == undefined) {
					return;
				}
				
				$this.unbind('.action');
				
				$this.bind('click.action',
					function () {
						if ($this.data('disable')) {
							return false;
						}
						$this.data('disable', true);
						
						if ($this.is('.default') || $this.find('img').length) {
							return true;
						}
						
						if ($form.is('.extra')) {
							tinymce.triggerSave();
						}
						
					
					var data = null;
						
						if (upload) {
							data = actionOnly ?
								new FormData()
								:
								new FormData($form[0])
							;
							
							if (id) {
								data.append('action', id.split('-').shift());
							} else {
								data.append('action', name);
								data.append(name, $this.attr('value'));
							}
						} else {
							data = (actionOnly ?
								''
								:
								$form.serialize()+'&'	
							) + (id ?
								'action='+id.split('-').shift()
								:
								'action='+name+'&'+name+'='+$this.attr('value')
							);
						}
								
						$.modal(
							'<div class="center processing">'+
								'<img src="/_images/process.gif" /> <br/>'+
								'Processing...'+
							'</div>'
						);
						
						window.onbeforeunload = function (event) {};
						
						$.ajax(
							{
								url: $form.attr('action'),
								type: 'POST',
								data: data,
								cache: false,
								processData: !upload,
								contentType: upload ? false : 'application/x-www-form-urlencoded; charset=UTF-8',
								dataType:  window.ActiveXObject ? 'text' : 'xml',
								context: $form,
								success: function (data) {
									$.modal();
									ajaxResponse(this, data);
									$this.data('disable', false);
								}
							}
						);
						return false;
					}
				);
			}
		);
	};
})(jQuery);

/**
 * State - AJAX onChange
 */
(function ($) {
	$AJAX = $.AjaxConveyor(
		{
			refresh: false
		}
	).
	
	before(
		function (request) {
			$AJAX.settings.refresh = $AJAX.settings.refresh || request.refresh;
		}
	).
	
	done(
		function () {
			if (!$AJAX.settings.refresh) {
				return;
			}
			$AJAX.settings.refresh = false;
			$AJAX.pipe({
				next: 'abort',
				ajax: {
					async: true,
					type: 'POST',
					data: 'action=run',
					cache: false,
					context: $('.content form'),
					success: function (data) {
						ajaxResponse(this, data);
					}
				}
			});
		}
	);

	$.fn.state = function () {
		return this.each(
			function (index, element) {
				var $this = $(element);
				var initial;
				
				$this.
					unbind('.state').
					
					bind('focus.state',
						function () {
							if (
								$this.is(':checkbox') ||
								$this.is(':radio') ||
								$this.is('select')
							) {
								return;
							}
							initial = $this.val();
						}
					).
					
					bind('blur.state',
						function () {
							if (
								$this.is(':checkbox') ||
								$this.is(':radio') ||
								$this.is('select')
							) {
								return;
							}
							if (initial != $this.val()) {
								$this.change();
							}
						}
					).
					
					bind('change.state',
						function () {
							var current = $this.val();
							
							if (initial == current) {
								return;
							}
							
							initial = current;
							
							var serialized = ($this.attr('for')) ?
								$this.serialize() + '&' + $('#'+$this.attr('for')).serialize()
								:
								($this.is(':checkbox') ?
									$('input[name="'+$this.attr('name')+'"]').serialize()
									:
									$this.serialize()
								)
							;
							
							var actionID = $this.attr('path') ?
								$this.attr('path')
								:
								$this.attr('id').split('|').shift()
							;
								
							$AJAX.pipe({
								refresh: $this.hasClass('ajax'),
								next: 'wait',
								ajax: {
									async: true,
									type: 'POST',
									data: serialized+'&action='+actionID+':change',
									cache: false,
									context: $(this.form)
								}
							});
						
						}
					)
				;
			}
		);
	};
})(jQuery);

/**
 * Help
 */
(function ($) {
	$.fn.help = function () {
		return this.each(
			function (index, element) {
				var $this = $(element);
				
				$this.
					unbind('.help').
					
					bind('click.help',
						function () {
							var $parent = $this.parent();
							if (!$parent.hasClass('field')) {
								return;
							}
							
							var id = $parent.find(':input').attr('id');
							if (id == undefined) {
								return;
							}
							id = id.split('|').shift();
							
							var $target = $parent.find('#'+escapeID(id));
							if ($target == undefined) {
								return;
							}
							
							$.ajax(
								'/',
								{
									type: 'POST',
									data: 'action=help&context='+id+'&id='+$target.val(),
									cache: false,
									context: $('#help'),
									success: function (data) {
										ajaxResponse(this, data);
									}
								}
							);
						}
					)
				;
			}
		);
	};
})(jQuery);
